import cguard

from cguard.access_manager import CasedGuardAccessManager
from cguard.requestor import GuardRequestor
from cguard.test.util import mock_response


requested_json = {
    "id": "guard_session_123",
    "state": "requested",
    "requester": {"id": "user_123", "email": "user@example.com"},
    "guard_application": {
        "id": "guard_application_abc",
        "name": "abc",
        "settings": {"message_of_the_day": None, "reason_required": True},
    },
}

approvial_json = {
    "id": "guard_session_123",
    "state": "approved",
    "requester": {"id": "user_123", "email": "user@example.com"},
    "guard_application": {
        "id": "guard_application_abc",
        "name": "abc",
        "settings": {"message_of_the_day": None, "reason_required": True},
    },
}

denial_json = {
    "id": "guard_session_123",
    "state": "denied",
    "requester": {"id": "user_123", "email": "user@example.com"},
    "guard_application": {
        "id": "guard_application_abc",
        "name": "abc",
        "settings": {"message_of_the_day": None, "reason_required": True},
    },
}


class TestAccessManager(object):
    def test_access_manager_creation(self):
        access_manager = CasedGuardAccessManager()
        assert access_manager.requestor.__class__ == GuardRequestor

    def test_access_manager_check_access_requested(self):
        with mock_response(json_data=requested_json):
            access_manager = CasedGuardAccessManager()

            assert (
                access_manager.check_access(
                    "app_123", "token_123", "session_abc", "user_abc"
                )
                == "requested"
            )

    def test_access_manager_check_access_approval(self):
        with mock_response(json_data=approvial_json):
            access_manager = CasedGuardAccessManager()

            assert (
                access_manager.check_access(
                    "app_123", "token_123", "session_abc", "user_abc"
                )
                == "approved"
            )

    def test_access_manager_check_access_denied(self):
        with mock_response(json_data=denial_json):
            access_manager = CasedGuardAccessManager()

            assert (
                access_manager.check_access(
                    "app_123", "token_123", "session_abc", "user_abc"
                )
                == "denied"
            )


import sys
import os
import getpass
import socket

from requests.exceptions import Timeout
from requests import get

from cguard.requestor.guard_requestor import GuardRequestor
from cguard.core.approval import Approval
from cguard.util import log_level, output, debug


class Client:
    def __init__(self, bin_name):
        self.bin_name = bin_name

    def _environment(self):
        username = getpass.getuser()
        hostname = socket.gethostname()

        return (username, hostname)

    def _hostname(self):
        return socket.gethostname()

    def _prompt_for_reason(self):
        output("Running under Cased Guard.")
        reason = input("[guard 🔒] Please enter a reason for access: ")
        return reason

    def _handle_response(
        self, res, command, app_name, program_args, app_token, user_token
    ):
        status_code = res.status_code
        body = res.json()

        if status_code == 200:
            # approved: either the session is active or auto-approval is on
            output("Running under Cased Guard.")

        elif status_code == 201:
            # approval required
            session_id = body.get("id")
            application_settings = body.get("guard_application").get("settings")
            waiting_message = (
                application_settings.get("message_of_the_day")
                or "Approval request sent.."
            )

            Approval().wait_for_approval(
                app_name, app_token, session_id, user_token, waiting_message
            )

        elif status_code == 400:
            # Either malformed or reason is required
            error = body.get("error")
            if error == "reason_required":
                reason = self._prompt_for_reason()
                self.execute(program_args, app_token, False, user_token, reason)
                return
            else:
                output("Request error: {}".format(error))
                debug(str(status_code) + " " + str(body))
                exit(1)

        elif status_code == 401:
            # Unauthorized
            output("Running under Cased Guard.")
            output("Access denied: {}".format(body.get("message")))

            if log_level() == "debug":
                debug(str(status_code) + " " + str(body))
            exit(0)

        elif status_code == 404:
            # App not found
            output(str(status_code) + " " + str(body))
            exit(0)

        elif status_code == 410:
            # Session request was cancelled
            output("Session request has already been cancelled.")
            exit(0)

        # call underlying
        os.system(command)

    def execute(
        self, program_args, app_token, deny_if_unreachable, user_token, reason=None
    ):
        username, hostname = self._environment()

        # Send request to API
        requestor = GuardRequestor()

        app_name = os.path.basename(os.path.normpath(self.bin_name))

        command = self.bin_name + " " + program_args

        # We wont have a session id (this is just an invocation of the program),
        # so we still need to request a new session. However, if we receive a
        # 200 (the server controls the session time) we'll be auto-approved.
        try:
            res = requestor.request_access(
                app_name, app_token, user_token, command, hostname, reason
            )
        except Timeout as e:
            if deny_if_unreachable == "1":
                output("Request timed-out. Per configuration, denying the request.")
                exit(0)
            else:
                output("Request timed-out. Per configuration, allowing the request.")
                os.system(command)

        if log_level() == "debug":
            debug(res.text)

        self._handle_response(
            res,
            command,
            app_name,
            program_args,
            app_token,
            user_token,
        )



import time

import time
import sys
import signal
from random import randint
from yaspin import yaspin

from cguard.access_manager import CasedGuardAccessManager
from cguard.requestor import GuardRequestor
from cguard.util import read_settings, poll_interval


class Approval:
    def __init__(self):
        self.access_manager = CasedGuardAccessManager()
        self.settings = read_settings()

    def wait_for_approval(
        self, app_name, app_token, session_id, user_token, waiting_message
    ):
        def signal_handler(sig, frame):
            print("\nExiting and cancelling request: {}\n".format(session_id))
            requestor = GuardRequestor()
            requestor.cancel_session(app_token, session_id, user_token)
            sys.exit(0)

        signal.signal(signal.SIGINT, signal_handler)

        wait_text = waiting_message + " (id: {})".format(session_id)
        with yaspin(text=wait_text, color="white") as spinner:
            while True:
                # poll the API for access granted
                state = self.access_manager.check_access(
                    app_name, app_token, session_id, user_token
                )
                if state == "approved":
                    msg = "✅ ACCESS APPROVED"
                    spinner.ok(msg)
                    break
                elif state == "denied":
                    spinner.fail("🛑 ACCESS DENIED")
                    exit(1)

                interval = poll_interval()
                time.sleep(interval)

        return True



try:
    from unittest import mock
except ImportError:
    import mock

import requests
import responses

import cguard

# HTTP Response Mock
def mock_response(
    status_code=200, content="response content", headers={}, json_data=None, **params
):
    mock_res = mock.Mock()
    mock_res.status_code = status_code
    mock_res.content = content
    mock_res.headers = headers

    if json_data:
        mock_res.json = mock.Mock(return_value=json_data)

    return mock.patch.object(
        cguard.requestor.HTTPClient, "make_request", return_value=mock_res
    )


def mock_requests(
    path, response={}, content_type="application/json", status=200, method=responses.GET
):
    """
    Mock the requests library.
    ONLY use this for testing the low-level HTTPClient itself — it should not be used
    for testing any guard API routes. Instead use mock_response().
    """
    url = "https://example.com" + path
    responses.add(method, url, json=response, status=status, content_type=content_type)



import requests
from os.path import expanduser, exists
import shutil
import hashlib

from cguard.util import home_dir, cguard_dir, log_level

DEFAULT_URL = "https://api.cased.com"


class HTTPClient:
    @classmethod
    def _cguard_hash(cls):
        filepath = shutil.which("cased")
        cguard = open(filepath, "rb")
        digest = hashlib.sha256(cguard.read()).hexdigest()
        cguard.close()
        return digest

    @classmethod
    def _check_for_server_error(cls, res):
        if res.status_code == 500:
            print(
                "Server error. Is a Slack channel configured for this Guard application?"
            )
            exit(1)

    @classmethod
    def make_request(cls, method, url, data=None, key=None):

        user_agent = "cased-guard"
        headers = {
            "User-Agent": user_agent,
            "X-Guard-Hash": cls._cguard_hash(),
            "Content-Type": "application/json",
        }

        if key:
            value = "Bearer " + key
            headers["Authorization"] = value

        if method == "get":
            res = requests.get(url, params=data, headers=headers, timeout=20)
        elif method == "post":
            res = requests.post(url, json=data, headers=headers, timeout=20)
        elif method == "put":
            res = requests.put(url, json=data, headers=headers, timeout=20)
        elif method == "patch":
            res = requests.patch(url, json=data, headers=headers, timeout=20)
        elif method == "head":
            res = requests.head(url, headers=headers, timeout=20)
        elif method == "delete":
            res = requests.delete(url, headers=headers, timeout=20)
        else:
            raise Exception(
                """Unsupported method given. This is likely a bug in the
                cased program. Please contact"""
            )

        cls._check_for_server_error(res)

        if log_level() == "debug":
            print(
                "Request sent. URL: {} | Params: {} | Headers: {}".format(
                    url, str(data), str(headers)
                )
            )

        return res


class GuardRequestor:
    def __init__(self, client=HTTPClient, url=None, *args, **kwargs):
        self.client = client
        self.url = url

    def _get_remote_url(self):
        final_url = None

        filepath = cguard_dir() + "/remote"
        if not exists(filepath):
            final_url = DEFAULT_URL
        else:
            with open(filepath, "r") as text_file:
                final_url = text_file.read()

        return final_url

    def _base_url(self):
        if self.url:
            return url
        else:
            return self._get_remote_url() + "/guard"

    def request_access(
        self, app_name, app_token, user_token, program_args, hostname, reason=None
    ):
        url = self._base_url() + "/sessions?user_token={}".format(user_token)
        data = {
            "guard_application_id": app_name,
            "command": program_args,
            "reason": reason,
            "metadata": {"hostname": hostname},
        }

        res = self.client.make_request("post", url, data=data, key=app_token)
        return res

    def get_session(self, app_name, app_token, session_id, user_token):
        url = self._base_url() + "/sessions/{}?user_token={}".format(
            session_id, user_token
        )
        data = {"guard_application_id": app_name}

        res = self.client.make_request("get", url, data=data, key=app_token)
        return res

    def cancel_session(self, app_token, session_id, user_token):
        url = self._base_url() + "/sessions/{}/cancel?user_token={}".format(
            session_id, user_token
        )

        res = self.client.make_request("post", url, key=app_token)
        return res

    def get_applications(self, user_token, environment):
        url = self._base_url() + "/applications?user_token={}&environment={}".format(
            user_token, environment
        )

        res = self.client.make_request("get", url)
        return res


import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


setup(
    name="cased",
    version="0.2.3",
    description="Cased Guard adds security and visibility features to the command line tools you and your team love.",
    long_description="Cased Guard adds security and visibility features to the command line tools you and your team love. https://cased.com/guard",
    author="Cased",
    author_email="support@cased.com",
    url="https://github.com/cased/guard-client",
    license="BSL 1.1",
    keywords="cased api",
    packages=find_packages(exclude=["tests", "tests.*"]),
    zip_safe=False,
    scripts=["cased-init", "cguard/cased"],
    python_requires=">3.5",
    install_requires=[
        "requests",
        "responses",
        "mock",
        "pytest >= 4.0c0",
        "pytest-mock",
        "pytest-xdist",
        "flask",
        "yaspin",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/cased/guard/issues",
        "Documentation": "https://docs.cased.com/guard",
        "Source Code": "https://github.com/cased/guard",
    },
)
